/*
!=======================================================================
!
!  PROGRAM  PHASE-Viewer  (PHASE-Viewer 2014.01 ver.3.3.0)
!
!  Created on 2006/10/16, 20:25
!  AUTHOR(S): KOGA, Junichiro
!  File : EkConvChecker.java
!  
!  Contact address :  Phase System Consortium
!                     E-mail: phase_system@nims.go.jp URL https://azuma.nims.go.jp
!
!
!   Since 2002, this program set had been intensively developed as a part of the following 
!  national projects supported by the Ministry of Education, Culture, Sports, Science and
!  Technology (MEXT) of Japan; "Frontier Simulation Software for Industrial Science
!  (FSIS)" from 2002 to 2005, "Revolutionary Simulation Software (RSS21)" from 2006 to
!  2008. "Research and Development of Innovative Simulation Software (RISS)" from 2008
!  to 2013. These projects is lead by the Center for Research on Innovative Simulation 
!  Software (CISS), the Institute of Industrial Science (IIS), the University of Tokyo.
!   Since 2013, this program set has been further developed centering on PHASE System
!  Consortium. 
!   The activity of development of this program set has been supervised by Takahisa Ohno.
!
!=======================================================================
 */

package ciss.phase_viewer.plugins.projectmanipulator.ekcal.inputcheck;

import org.apache.log4j.Logger;

import ciss.phase_viewer.input.PhaseInputChecker;

/**
 * ekcal(epsmain)ŗp, ek_convergenceubN̑Ó`FbN
 * 
 * @author
 */
public class EkConvChecker extends PhaseInputChecker {
    private Logger logger = Logger.getLogger(ChargeChecker.class.getName());

    /** Creates a new instance of ChargeChecker */
    public EkConvChecker() {
    }

    public boolean isFixable() {
        return true;
    }

    public String getName() {
        return "ek_convergence";
    }

    public String getDescription() {
        return "check the 'ek_convergence' block.";
    }

    public String getCandidate() {
        String ret = "";
        if (state != OK)
            ret = "will set delta_eigenvalue to 1.0e-5 hartree";
        return ret;
    }

    public boolean fixInput() {
        fixPrimitiveEntry("accuracy.ek_convergence", "delta_eigenvalue",
                "1e-5", "Hartree");
        return true;
    }

    private double defaultDelta = 1.0e-5;

    public int doCheck() {
        String delta_eigen = inputInterface.getInputInterfacePrimitiveEntry(
                "accuracy.ek_convergence.delta_eigenvalue").getValue();
        double del = 0;
        state = OK;
        message = "";
        try {
            del = Double.parseDouble(delta_eigen);
            if (del < 0) {
                this.state = ERROR;
                message = "delta_eigenvalue is negative";
            } else if (del < 1.0e-8) {
                message = "delta_eigenvalue is probably too small;" + LS;
                message += "your calculation may not converge";
                this.state = WARNING;
            }
        } catch (NumberFormatException e) {
            message = "delta_eigenvalue is probably too small;" + LS;
            message += "your calculation will probably not converge";
            this.state = WARNING;
        }

        return state;
    }

}
